@@ -34,6 +34,7 @@ const DesktopActions = class {
         this._Enums = desktopManager.Enums;
         this._desktopMonitor = desktopManager.desktopMonitor;
         this._windowManager = desktopManager.windowManager;
+        this.lastAnchorSelected = null;
         this._isCut = false;
         this._clipboardFiles = null;
         this._intDBusSignalMonitoring();
@@ -232,7 +233,7 @@ const DesktopActions = class {
         const findFilesAction = Gio.SimpleAction.new('findFiles', null);
         findFilesAction.connect(
             'activate',
-            () =>  this._desktopManager.findFiles(null)
+            () => this._desktopManager.findFiles(null)
         );
         this._mainApp.add_action(findFilesAction);
 
@@ -283,6 +284,13 @@ const DesktopActions = class {
         });
         this._mainApp.add_action(previewAction);
 
+        const toggleKeyboardSelection =
+            Gio.SimpleAction.new('toggleKeyboardSelection', null);
+        toggleKeyboardSelection.connect('activate', () => {
+            this._toggleKeyboardSelection();
+        });
+        this._mainApp.add_action(toggleKeyboardSelection);
+
         const chooseIconLeft = Gio.SimpleAction.new('chooseIconLeft', null);
         chooseIconLeft.connect('activate', () => {
             this._selectFileItemInDirection(Gdk.KEY_Left);
@@ -531,6 +539,7 @@ const DesktopActions = class {
             this.doRedoSimpleAction.set_enabled(false);
             return;
         }
+
         switch (this._DBusUtils.RemoteFileOperations.UndoStatus()) {
         case this._Enums.UndoStatus.UNDO:
             this.doUndoSimpleAction.set_enabled(true);
@@ -659,43 +668,33 @@ const DesktopActions = class {
     }
 
     _selectFileItemInDirection(symbol) {
+        // Anchoring
         var index;
         var multiplier;
-        let selection = this.currentSelection;
-        if (!selection) {
+        const previousSelection = this.currentSelection;
+        let selection = previousSelection;
+
+        if (!selection || selection.length === 0) {
             if (this.activeFileItem && this.activeFileItem.isStackMarker)
                 selection = [this.activeFileItem];
             else
                 selection = this._displayList;
         }
-        if (!selection)
+
+        if (!selection || selection.length === 0)
             return false;
 
-        let selected = selection[0];
-        let selectedCoordinates = selected.getCoordinates();
-        if (!this.isShift)
-            this._desktopManager.unselectAll();
-        if (selection.length > 1) {
-            for (let item of selection) {
-                let itemCoordinates = item.getCoordinates();
-                if (itemCoordinates[0] > selectedCoordinates[0])
-                    continue;
+        if (!this.keyboardSelected)
+            this._setKeyboardSelected(selection[0]);
 
-                if (symbol === Gdk.KEY_Down || symbol === Gdk.KEY_Right) {
-                    if ((itemCoordinates[0] > selectedCoordinates[0]) ||
-                        (itemCoordinates[1] > selectedCoordinates[1])) {
-                        selected = item;
-                        selectedCoordinates = itemCoordinates;
-                        continue;
-                    }
-                } else if ((itemCoordinates[0] < selectedCoordinates[0]) ||
-                        (itemCoordinates[1] < selectedCoordinates[1])) {
-                    selected = item;
-                    selectedCoordinates = itemCoordinates;
-                    continue;
-                }
-            }
-        }
+        let selected = this.keyboardSelected;
+
+        if (!selected)
+            return false;
+
+        let selectedCoordinates = selected.getCoordinates();
+
+        // Navigation
         switch (symbol) {
         case Gdk.KEY_Left:
             index = 0;
@@ -713,38 +712,146 @@ const DesktopActions = class {
             index = 1;
             multiplier = 1;
             break;
+        default:
+            return false;
         }
-        let newDistance = null;
+
+        const selectedCenterX =
+            (selectedCoordinates[0] + selectedCoordinates[2]) / 2;
+        const selectedCenterY =
+            (selectedCoordinates[1] + selectedCoordinates[3]) / 2;
+
+        let bestScore = null;
         let newItem = null;
+
+        let wrapItem = null;
+        let wrapExtreme = null;
+        let wrapSecondary = null;
+
         for (let item of this._displayList) {
-            let itemCoordinates = item.getCoordinates();
-            if ((selectedCoordinates[index] * multiplier) >=
-                (itemCoordinates[index] * multiplier))
+            if (item === selected)
                 continue;
 
-            let distance =
-                Math.pow(
-                    selectedCoordinates[0] - itemCoordinates[0], 2) +
-                Math.pow(
-                    selectedCoordinates[1] -  itemCoordinates[1], 2);
+            let itemCoordinates = item.getCoordinates();
+            const itemCenterX = (itemCoordinates[0] + itemCoordinates[2]) / 2;
+            const itemCenterY = (itemCoordinates[1] + itemCoordinates[3]) / 2;
 
-            if ((newDistance === null) || (newDistance > distance)) {
-                newDistance = distance;
-                newItem = item;
+            const deltaX = itemCenterX - selectedCenterX;
+            const deltaY = itemCenterY - selectedCenterY;
+
+            const primary = index === 0 ? deltaX : deltaY;
+            const secondary = index === 0 ? Math.abs(deltaY) : Math.abs(deltaX);
+
+            // Forward candidates (in the requested direction).
+            if (primary * multiplier > 0) {
+                const score = Math.abs(primary) + secondary;
+                if ((bestScore === null) || (score < bestScore)) {
+                    bestScore = score;
+                    newItem = item;
+                }
+                continue;
+            }
+
+            // Wrap candidate: farthest in the opposite direction,
+            // with row/col bias.
+            if (wrapExtreme === null ||
+                (multiplier > 0
+                    ? primary < wrapExtreme
+                    : primary > wrapExtreme
+                ) ||
+                (primary === wrapExtreme && secondary < wrapSecondary)
+            ) {
+                wrapExtreme = primary;
+                wrapSecondary = secondary;
+                wrapItem = item;
             }
         }
-        if (newItem === null)
-            newItem = selected;
 
-        newItem.setSelected();
-        if (newItem.isStackMarker)
-            newItem.keyboardSelected();
+        if (newItem === null)
+            newItem = wrapItem || selected;
+
+        // Selection logic
+        const {ctrl, shift} = this._desktopManager.modifierMode;
+        const keptSelection = previousSelection || [];
+
+        if (shift && selected) {
+        // Shift: select everything in the rectangle between anchor and focus,
+        // extending (not clearing) the existing selection.
+            const anchor = this.lastAnchorSelected &&
+                this._displayList.includes(this.lastAnchorSelected)
+                ? this.lastAnchorSelected
+                : selected;
+            const focusItem = anchor !== selected ? selected : newItem;
+            const sRect = anchor.iconRectangle;
+            const nRect = focusItem.iconRectangle;
+            const minX = Math.min(sRect.x, nRect.x);
+            const maxX = Math.max(
+                sRect.x + sRect.width,
+                nRect.x + nRect.width
+            );
+            const minY = Math.min(sRect.y, nRect.y);
+            const maxY = Math.max(
+                sRect.y + sRect.height,
+                nRect.y + nRect.height
+            );
+
+            this._displayList.forEach(item => {
+                const rect = item.iconRectangle;
+                const withinX = rect.x <= maxX && rect.x + rect.width >= minX;
+                const withinY = rect.y <= maxY && rect.y + rect.height >= minY;
+
+                if (withinX && withinY)
+                    item.setSelected();
+            });
+
+            // Keep any prior selection intact
+            keptSelection.forEach(item => item.setSelected());
+            focusItem.setSelected();
+            this.lastAnchorSelected = focusItem;
+
+            if (anchor !== selected) {
+                // Cancel navigation when extending from a previous anchor
+                this._setKeyboardSelected(selected);
+                this._desktopManager.fileItemMenu.activeFileItem = selected;
+                this.activeFileItem = selected;
+                return true;
+            }
+        } else if (ctrl) {
+            // Ctrl: do not alter existing selection
+            if (newItem.isSelected)
+                this.lastAnchorSelected = newItem;
+        } else {
+            // Default: move selection to the new item only
+            this._desktopManager.unselectAll();
+            newItem.setSelected();
+            this.lastAnchorSelected = newItem;
+        }
+
+        // Always keyboard-focus the new item
+        this._setKeyboardSelected(newItem);
 
         this._desktopManager.fileItemMenu.activeFileItem = newItem;
         this.activeFileItem = newItem;
+
         return true;
     }
 
+    _toggleKeyboardSelection() {
+        const item = this.keyboardSelected;
+        if (!item)
+            return;
+
+        if (item.isSelected)
+            item.unsetSelected();
+        else
+            item.setSelected();
+    }
+
+    _setKeyboardSelected(fileItem) {
+        this._displayList.forEach(f => f.keyboardUnSelected());
+        fileItem.keyboardSelected();
+    }
+
     _menuKeyPressed() {
         const selection = this.currentSelection;
         if (selection) {
@@ -768,11 +875,10 @@ const DesktopActions = class {
                 false,
                 false,
                 grid[0]
-            ).catch(e => console.error(e));
+            );
         }
     }
 
-
     async _newDocument(template) {
         if (!template)
             return;
@@ -846,6 +952,19 @@ const DesktopActions = class {
         return this._fileItemMenu.activeFileItem;
     }
 
+    get keyboardSelected() {
+        let keyboardSelectedItem = null;
+
+        for (let fileItem of this._displayList) {
+            if (fileItem.KeyboardSelected) {
+                keyboardSelectedItem = fileItem;
+                break;
+            }
+        }
+
+        return keyboardSelectedItem;
+    }
+
     set activeFileItem(fileItem) {
         this._fileItemMenu.activeFileItem = fileItem;
     }
@@ -1013,6 +1132,14 @@ const DesktopBackgroundMenu = class {
 
         this.desktopBackgroundGioMenu.append_section(null, settingsMenu);
 
+        if (this._Prefs.showDesktopWidgets) {
+            const widgetLayerMenu = Gio.Menu.new();
+            widgetLayerMenu.append(
+                _('Edit Widgets…'), 'app.toggleWidgetLayer');
+
+            this.desktopBackgroundGioMenu.append_section(null, widgetLayerMenu);
+        }
+
         const backgroundMenu = Gio.Menu.new();
         backgroundMenu.append(
             _('Shell Menu…'), 'app.displayShellBackgroundMenu');
@@ -1051,14 +1178,15 @@ const DesktopBackgroundMenu = class {
 
         this.popupmenu.set_has_arrow(false);
         this.popupmenu.popup();
-        this.popupmenu.connect('closed', async () => {
-            await this._waitDelayMs(50);
-            this.popupmenu.unparent();
-            this.popupmenu = null;
-            if (this.popupmenuclosed)
-                this.popupmenuclosed(true);
-            this.popupmenuclosed = null;
+        this.popupmenu.connect('closed', () => {
+            GLib.idle_add(GLib.PRIORITY_DEFAULT_IDLE, () => {
+                this.popupmenu.unparent();
+                this.popupmenu = null;
+                if (this.popupmenuclosed)
+                    this.popupmenuclosed(true);
+                this.popupmenuclosed = null;
+                return GLib.SOURCE_REMOVE;
+            });
         });
     }
 };
-
