@@ -77,13 +77,13 @@ const DragManager = class {
             let i = 0;
             let newSymlinkName = fileGio.get_basename();
             let checkSymlinkGio;
+
             do {
                 checkSymlinkGio =
-                    Gio.File.new_for_commandline_arg(
-                        GLib.build_filenamev(
-                            [gioDestination.get_path(), newSymlinkName]
-                        )
+                    Gio.File.new_build_filenamev(
+                        [gioDestination.get_path(), newSymlinkName]
                     );
+
                 try {
                     checkSymlinkGio.make_symbolic_link(
                         GLib.build_filenamev([fileGio.get_path()]),
@@ -271,9 +271,9 @@ const DragManager = class {
         return false;
     }
 
-    _drawSelectionRectangles() {
+    _drawRubberBand() {
         for (let grid of this._desktops)
-            grid.drawRubberBand();
+            grid.updateOverlay();
     }
 
     // Global Methods
@@ -365,15 +365,16 @@ const DragManager = class {
         this.pendingSelfCopyFiles = {};
 
         await Promise.all(fileList.map(async element => {
-            let file = Gio.File.new_for_uri(element);
+            const file = Gio.File.new_for_uri(element);
 
             if (!file.is_native()) {
                 this.setPendingDropCoordinates(file, dropCoordinates);
                 return;
             }
 
-            let info = new Gio.FileInfo();
+            const info = new Gio.FileInfo();
             info.set_attribute_string('metadata::desktop-icon-position', '');
+
             if (dropCoordinates !== null) {
                 if (!opts.doCopy) {
                     info.set_attribute_string(
@@ -535,6 +536,7 @@ const DragManager = class {
                             {doCopy: forceCopy}
                         );
                     }
+
                     returnAction = await this.copyOrMoveUris(
                         fileList,
                         this._desktopDir.get_uri(),
@@ -549,6 +551,7 @@ const DragManager = class {
                     returnAction = Gdk.DragAction.LINK;
                 else
                     returnAction = Gdk.DragAction.COPY;
+
                 this.askWhatToDoWithFiles(
                     fileList,
                     this._desktopDir.get_uri(),
@@ -637,7 +640,7 @@ const DragManager = class {
                     'width': this.x2 - this.x1,
                     'height': this.y2 - this.y1,
                 });
-            this._drawSelectionRectangles();
+            this._drawRubberBand();
             for (let item of this._displayList) {
                 const labelintersect =
                     item.labelRectangle.intersect(this.selectionRectangle)[0];
@@ -659,7 +662,7 @@ const DragManager = class {
             this.selectionRectangle = null;
         }
         for (let grid of this._desktops)
-            grid.drawRubberBand();
+            grid.updateOverlay();
 
         return false;
     }
@@ -671,6 +674,7 @@ const DragManager = class {
     }
 
     selected(fileItem, action) {
+        this._clearKeyboardSelection();
         switch (action) {
         case this._Enums.Selection.ALONE:
             if (!fileItem.isSelected) {
@@ -713,6 +717,11 @@ const DragManager = class {
         }
     }
 
+    _clearKeyboardSelection() {
+        this._displayList.forEach(item => item.keyboardUnSelected());
+        this._desktopManager.desktopActions.lastAnchorSelected = null;
+    }
+
     // File Copy Move Link Methods
 
     async copyOrMoveUris(uriList, destinationUri, event, params = {}) {
@@ -879,19 +888,21 @@ const DragManager = class {
 
     async makeLinks(fileList, destination, X, Y) {
         const gioDestination = Gio.File.new_for_uri(destination);
+
         await Promise.all(fileList.map(async file => {
             const fileGio = Gio.File.new_for_uri(file);
+
             const newSymlinkName =
                 this._desktopManager.desktopMonitor
                 .getDesktopUniqueFileName(
                     fileGio.get_basename()
                 );
+
             const symlinkGio =
-                Gio.File.new_for_commandline_arg(
-                    GLib.build_filenamev(
-                        [gioDestination.get_path(), newSymlinkName]
-                    )
+                Gio.File.new_build_filenamev(
+                    [gioDestination.get_path(), newSymlinkName]
                 );
+
             try {
                 const linkMade =
                     symlinkGio.make_symbolic_link(
@@ -901,10 +912,12 @@ const DragManager = class {
 
                 if (linkMade) {
                     const info = new Gio.FileInfo();
+
                     info.set_attribute_string(
                         'metadata::nautilus-drop-position',
                         `${X},${Y}`
                     );
+
                     info.set_attribute_string(
                         'metadata::desktop-icon-position',
                         ''
@@ -972,4 +985,3 @@ const DragManager = class {
         return this._localDrag();
     }
 };
-
