@@ -16,7 +16,7 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-import {Gio, GLib, Gdk, DesktopAppInfo} from '../../dependencies/gi.js';
+import {Gio, GLib, Gdk, GdkX11, DesktopAppInfo} from '../../dependencies/gi.js';
 import {_} from '../../dependencies/gettext.js';
 
 export {DesktopIconsUtil};
@@ -39,9 +39,7 @@ const DesktopIconsUtil = class {
 
 
     usingX11() {
-        return Gdk.Display.get_default()
-            .constructor
-            .$gtype.name === 'GdkX11Display';
+        return Gdk.Display.get_default() instanceof GdkX11.X11Display;
     }
 
     ensureDir(path) {
@@ -60,26 +58,19 @@ const DesktopIconsUtil = class {
      * Returns the Nautilus scripts directory as a Gio.File
      */
     getScriptsDir() {
-        const scriptsDir =
-            GLib.build_filenamev(
-                [GLib.get_home_dir(), this.Enums.NAUTILUS_SCRIPTS_DIR]
-            );
-
-        return Gio.File.new_for_commandline_arg(scriptsDir);
+        return Gio.File.new_build_filenamev([
+            GLib.get_home_dir(),
+            this.Enums.NAUTILUS_SCRIPTS_DIR,
+        ]);
     }
 
     getUserTerminalConfFile() {
         const xdgUserConfigFolder = GLib.get_user_config_dir();
 
-        const xdgUserTerminalListFile =
-            GLib.build_filenamev(
-                [
-                    xdgUserConfigFolder,
-                    this.Enums.XDG_TERMINAL_LIST_FILE,
-                ]
-            );
-
-        return Gio.File.new_for_commandline_arg(xdgUserTerminalListFile);
+        return Gio.File.new_build_filenamev([
+            xdgUserConfigFolder,
+            this.Enums.XDG_TERMINAL_LIST_FILE,
+        ]);
     }
 
     getSystemTerminalConfFile() {
@@ -87,15 +78,11 @@ const DesktopIconsUtil = class {
         const systemTerminalConfFiles = [];
 
         xdgEtcConfigFolder.forEach(f => {
-            const xdgSystemTerminalListFile = GLib.build_filenamev(
-                [
+            const gioFile =
+                Gio.File.new_build_filenamev([
                     f,
                     this.Enums.XDG_TERMINAL_LIST_FILE,
-                ]
-            );
-
-            const gioFile =
-                Gio.File.new_for_commandline_arg(xdgSystemTerminalListFile);
+                ]);
 
             systemTerminalConfFiles.push(gioFile);
         });
@@ -106,14 +93,11 @@ const DesktopIconsUtil = class {
     getUserDataTerminalDir() {
         const userDataDir = GLib.get_user_data_dir();
 
-        const terminalDir =
-            GLib.build_filenamev([
-                userDataDir,
-                this.Enums.XDG_TERMINAL_DIR,
-                this.Enums.XDG_TERMINAL_LIST_FILE,
-            ]);
-
-        return Gio.File.new_for_commandline_arg(terminalDir);
+        return Gio.File.new_build_filenamev([
+            userDataDir,
+            this.Enums.XDG_TERMINAL_DIR,
+            this.Enums.XDG_TERMINAL_LIST_FILE,
+        ]);
     }
 
     getSystemDataTerminalDirs() {
@@ -121,12 +105,11 @@ const DesktopIconsUtil = class {
         const systemDataTerminalFiles = [];
 
         systemDataDirs.forEach(f => {
-            const file = GLib.build_filenamev([
+            const gioFile = Gio.File.new_build_filenamev([
                 f,
                 this.Enums.XDG_TERMINAL_DIR,
                 this.Enums.XDG_TERMINAL_LIST_FILE,
             ]);
-            const gioFile = Gio.File.new_for_commandline_arg(file);
             systemDataTerminalFiles.push(gioFile);
         });
 
@@ -163,8 +146,7 @@ const DesktopIconsUtil = class {
         if (!appId)
             throw new Error('Application ID is not available');
 
-        const appDirPath = GLib.build_filenamev([userDataDir, appId]);
-        return Gio.File.new_for_commandline_arg(appDirPath);
+        return Gio.File.new_build_filenamev([userDataDir, appId]);
     }
 
     /**
@@ -815,16 +797,12 @@ const DesktopIconsUtil = class {
         return new Promise((resolve, reject) => {
             let gioFile = Gio.File.new_for_uri(fileUri);
 
-            let destinationGioFile = Gio.File.new_for_path(
-                GLib.build_filenamev(
-                    [
-                        GLib.get_user_special_dir(
-                            GLib.UserDirectory.DIRECTORY_DESKTOP
-                        ),
-                        gioFile.get_basename(),
-                    ]
-                )
-            );
+            let destinationGioFile = Gio.File.new_build_filenamev([
+                GLib.get_user_special_dir(
+                    GLib.UserDirectory.DIRECTORY_DESKTOP
+                ),
+                gioFile.get_basename(),
+            ]);
 
             let gioFileCopyFlags =
                 Gio.FileCopyFlags.OVERWRITE |
