@@ -77,10 +77,13 @@ const WidgetManager = class {
 
         // When true, suppress emitting stateChanged events
         this._suppressStateEvents = false;
+        this._loadStatePromise = null;
+        this._pendingLoadState = null;
 
         this._addActions();
 
-        this.loadState(this._preferences.widgetState).catch(e => logError(e));
+        // loadState is handled during startup and by Preferences; avoid
+        // overlapping loads during construction.
     }
 
     clearFromGrids() {
@@ -152,8 +155,19 @@ const WidgetManager = class {
             return;
 
         this._updateAddWidgetButtonVisibility(surface, onTop);
+        this._updateGridToggleButtonVisibility(surface, onTop);
         this._raiseAddButton(surface);
+        this._raiseGridToggleButton(surface);
         this._updateWidgetLayerChange(monitorIndex, onTop);
+
+        if (!onTop) {
+            if (surface.gridToggleButton) {
+                surface.gridToggleButton.set_active(false);
+            }
+            
+            surface.grid.widgetGridEnabled = false;
+            surface.grid.updateOverlay();
+        }
     }
 
     // =====================================================================
@@ -368,21 +382,30 @@ const WidgetManager = class {
         const h = inst.height;
 
         // Clamp to stay inside the grid's usable area
+        let clamped = false;
         if (wNorm > 0 && w <= wNorm) {
-            if (x + w > wNorm)
+            if (x + w > wNorm) {
                 x = wNorm - w;
-            if (x < 0)
+                clamped = true;
+            }
+            if (x < 0) {
                 x = 0;
+                clamped = true;
+            }
         }
 
         if (hNorm > 0 && h <= hNorm) {
-            if (y + h > hNorm)
+            if (y + h > hNorm) {
                 y = hNorm - h;
-            if (y < 0)
+                clamped = true;
+            }
+            if (y < 0) {
                 y = 0;
+                clamped = true;
+            }
         }
 
-        return {x, y, width: w, height: h};
+        return {x, y, width: w, height: h, clamped};
     }
 
     get instances() {
@@ -432,7 +455,7 @@ const WidgetManager = class {
         }
 
         const inst = this._instances.get(instanceId);
-        if (!inst?.actor || inst._isAddButton) {
+        if (!inst?.actor || inst._isAddButton || inst._isGridToggleButton) {
             this._selectedInstanceId = null;
             this._detachChrome();
             this._updateWidgetsSelectionState();
@@ -556,6 +579,26 @@ const WidgetManager = class {
      * It also has to deal with null, undefined, or missing fields gracefully.
      */
     async loadState(state) {
+        if (this._loadStatePromise) {
+            this._pendingLoadState = state;
+            return this._loadStatePromise;
+        }
+
+        this._loadStatePromise = this._loadStateInner(state);
+
+        try {
+            await this._loadStatePromise;
+        } finally {
+            this._loadStatePromise = null;
+            if (this._pendingLoadState) {
+                const pending = this._pendingLoadState;
+                this._pendingLoadState = null;
+                await this.loadState(pending);
+            }
+        }
+    }
+
+    async _loadStateInner(state) {
         if (!state || typeof state !== 'object')
             return;
 
@@ -577,87 +620,109 @@ const WidgetManager = class {
         const prevSelection = this._selectedInstanceId;
 
         // Avoid emitting stateChanged while rebuilding from persisted state.
-        // Restore the previous suppression flag afterward.
+        // Only suppress around positioning, so we don't block saves for long
+        // async operations (e.g., consent prompts).
         const previousSuppressionState = this._suppressStateEvents;
-        this._suppressStateEvents = true;
 
-        const seen = new Set();
+        try {
+            const seen = new Set();
 
-        for (const instData of state.instances) {
-            if (!instData.instanceId || !instData.widgetId)
-                continue;
-
-            let instance = this._instances.get(instData.instanceId);
-
-            if (instance) {
-                instance.widgetId = instData.widgetId;
-                instance.monitorIndex = instData.monitorIndex ?? 0;
-                instance.kind = instData.kind ?? 'html';
-                instance.normX = instData.normX ?? 0;
-                instance.normY = instData.normY ?? 0;
-                instance.width = instData.width ?? 200;
-                instance.height = instData.height ?? 150;
-                instance.config = instData.config ?? {};
-                instance.prefsUri = instData.prefsUri ?? null;
-                instance.hasPreferences =
-                    instData.hasPreferences ?? !!instance.prefsUri;
-                instance.hasBackend = instData.hasBackend;
-                instance.webConsent = instData.webConsent ?? null;
-                instance.backendConsent = instData.backendConsent ?? null;
-            } else {
-                instance = {
-                    instanceId: instData.instanceId,
-                    widgetId: instData.widgetId,
-                    monitorIndex: instData.monitorIndex ?? 0,
-                    kind: instData.kind ?? 'html',
-                    normX: instData.normX ?? 0,
-                    normY: instData.normY ?? 0,
-                    width: instData.width ?? 200,
-                    height: instData.height ?? 150,
-                    actor: null,
-                    config: instData.config ?? {},
-                    prefsUri: instData.prefsUri ?? null,
-                    hasPreferences:
-                        instData.hasPreferences ?? !!instData.prefsUri,
-                    hasBackend: instData.hasBackend ?? false,
-                    webConsent: instData.webConsent ?? null,
-                    backendConsent: instData.backendConsent ?? null,
-                };
-
-                this._instances.set(instance.instanceId, instance);
-            }
-
-            seen.add(instance.instanceId);
-
-            const surface = this._surfaces.get(instance.monitorIndex);
-            if (surface) {
-                // eslint-disable-next-line no-await-in-loop
-                const created = await this._ensureInstanceActor(instance);
-
-                if (!created)
+            for (const instData of state.instances) {
+                if (!instData.instanceId || !instData.widgetId)
                     continue;
 
-                this._positionInstanceActor(instance);
+                try {
+                    let instance = this._instances.get(instData.instanceId);
+
+                    if (instance) {
+                        instance.widgetId = instData.widgetId;
+                        instance.monitorIndex = instData.monitorIndex ?? 0;
+                        instance.kind = instData.kind ?? 'html';
+                        instance.normX = instData.normX ?? 0;
+                        instance.normY = instData.normY ?? 0;
+                        instance.width = instData.width ?? 200;
+                        instance.height = instData.height ?? 150;
+                        instance.config = instData.config ?? {};
+                        instance.prefsUri = instData.prefsUri ?? null;
+                        instance.hasPreferences =
+                            instData.hasPreferences ?? !!instance.prefsUri;
+                        instance.hasBackend = instData.hasBackend;
+                        instance.webConsent = instData.webConsent ?? null;
+                        instance.backendConsent =
+                            instData.backendConsent ?? null;
+                    } else {
+                        instance = {
+                            instanceId: instData.instanceId,
+                            widgetId: instData.widgetId,
+                            monitorIndex: instData.monitorIndex ?? 0,
+                            kind: instData.kind ?? 'html',
+                            normX: instData.normX ?? 0,
+                            normY: instData.normY ?? 0,
+                            width: instData.width ?? 200,
+                            height: instData.height ?? 150,
+                            actor: null,
+                            config: instData.config ?? {},
+                            prefsUri: instData.prefsUri ?? null,
+                            hasPreferences:
+                                instData.hasPreferences ?? !!instData.prefsUri,
+                            hasBackend: instData.hasBackend ?? false,
+                            webConsent: instData.webConsent ?? null,
+                            backendConsent: instData.backendConsent ?? null,
+                        };
+
+                        this._instances.set(instance.instanceId, instance);
+                    }
+
+                    seen.add(instance.instanceId);
+
+                    const surface = this._surfaces.get(instance.monitorIndex);
+                    if (surface) {
+                        // eslint-disable-next-line no-await-in-loop
+                        const created =
+                            await this._ensureInstanceActor(instance);
+
+                        if (!created)
+                            continue;
+
+                        const prev = this._suppressStateEvents;
+                        this._suppressStateEvents = true;
+                        this._positionInstanceActor(instance);
+                        this._suppressStateEvents = prev;
+                    }
+                } catch (e) {
+                    console.error(
+                        'WidgetManager loadState failed for instance:',
+                        {
+                            instanceId: instData.instanceId,
+                            widgetId: instData.widgetId,
+                            monitorIndex: instData.monitorIndex,
+                            kind: instData.kind,
+                        },
+                        e
+                    );
+                }
             }
+
+            for (const instanceId of [...this._instances.keys()]) {
+                const instance = this._instances.get(instanceId);
+                if (instance?._isAddButton || instance?._isGridToggleButton)
+                    continue;
+
+                if (seen.has(instanceId))
+                    continue;
+
+                this._removeActor(instanceId);
+            }
+
+            if (prevSelection && this._instances.has(prevSelection))
+                this.selectInstance(prevSelection);
+            else
+                this.selectInstance(null);
+        } catch (e) {
+            console.error('WidgetManager loadState failed:', e);
+        } finally {
+            this._suppressStateEvents = previousSuppressionState;
         }
-
-        for (const instanceId of [...this._instances.keys()]) {
-            const instance = this._instances.get(instanceId);
-            if (instance?._isAddButton)
-                continue;
-
-            if (seen.has(instanceId))
-                continue;
-
-            this._removeActor(instanceId);
-        }
-
-        if (prevSelection && this._instances.has(prevSelection))
-            this.selectInstance(prevSelection);
-        else
-            this.selectInstance(null);
-
-        this._suppressStateEvents = previousSuppressionState;
     }
 
     updateInstanceConfig(instanceId, newConfig) {
@@ -706,7 +771,7 @@ const WidgetManager = class {
                 const instanceId = child.widgetInstanceId;
                 if (instanceId) {
                     const inst = this._instances.get(instanceId);
-                    if (inst && !inst._isAddButton) {
+                    if (inst && !inst._isAddButton && !inst._isGridToggleButton) {
                         if (!zIndexByInstanceId.has(instanceId))
                             zIndexByInstanceId.set(instanceId, i);
                     }
@@ -737,7 +802,7 @@ const WidgetManager = class {
         const instances = [];
 
         for (const inst of this._instances.values()) {
-            if (inst._isAddButton)
+            if (inst._isAddButton || inst._isGridToggleButton)
                 continue;
 
             instances.push(inst);
@@ -905,10 +970,13 @@ const WidgetManager = class {
     }
 
     _rebuildSurfacesFrom(desktops) {
-        const existingButtons = new Map();
+        const existingAddButtons = new Map();
+        const existingGridToggleButtons = new Map();
         for (const inst of this._instances.values()) {
             if (inst._isAddButton)
-                existingButtons.set(inst.monitorIndex, inst);
+                existingAddButtons.set(inst.monitorIndex, inst);
+            else if (inst._isGridToggleButton)
+                existingGridToggleButtons.set(inst.monitorIndex, inst);
         }
 
         for (const surface of this._surfaces.values())
@@ -938,12 +1006,16 @@ const WidgetManager = class {
                 widgetContainer,
                 monitorIndex,
                 addButton: null,
+                gridToggleButton: null,
             };
 
             this._surfaces.set(monitorIndex, surface);
 
-            const existingInst = existingButtons.get(monitorIndex);
-            this._ensureAddWidgetButton(surface, existingInst);
+            const existingAddInst = existingAddButtons.get(monitorIndex);
+            this._ensureAddWidgetButton(surface, existingAddInst);
+            const existingGridToggleInst =
+                existingGridToggleButtons.get(monitorIndex);
+            this._ensureGridToggleButton(surface, existingGridToggleInst);
         }
     }
 
@@ -968,6 +1040,24 @@ const WidgetManager = class {
             addInst.actor = null;
         else if (addButtonInstanceId)
             this._instances.delete(addButtonInstanceId);
+
+        if (surface.gridToggleButton) {
+            const parent = surface.gridToggleButton.get_parent?.();
+            if (parent?.remove)
+                parent.remove(surface.gridToggleButton);
+
+            surface.gridToggleButton = null;
+        }
+
+        const gridToggleButtonInstanceId = 
+            this._getGridToggleButtonInstanceId(surface.monitorIndex);
+        const gridToggleInst = gridToggleButtonInstanceId 
+            ? this._instances.get(gridToggleButtonInstanceId) 
+            : null;
+        if (gridToggleInst?._isGridToggleButton)
+            gridToggleInst.actor = null;
+        else if (gridToggleButtonInstanceId)
+            this._instances.delete(gridToggleButtonInstanceId);
     }
 
     _getAddButtonInstanceId(monitorIndex) {
@@ -976,6 +1066,12 @@ const WidgetManager = class {
         return `__ding-add-button-${monitorIndex}`;
     }
 
+    _getGridToggleButtonInstanceId(monitorIndex) {
+        if (monitorIndex === undefined || monitorIndex === null)
+            return null;
+        return `__ding-widget-grid-toggle-button-${monitorIndex}`;
+    }
+
     _ensureAddWidgetButton(surface, existingInst = null) {
         if (!surface?.widgetContainer)
             return;
@@ -1015,8 +1111,8 @@ const WidgetManager = class {
             kind: 'chrome',
             normX: 0,
             normY: 0,
-            width: 64,
-            height: 64,
+            width: 48,
+            height: 48,
             actor: button,
             config: {},
             _isAddButton: true,
@@ -1040,6 +1136,69 @@ const WidgetManager = class {
         this._raiseAddButton(surface);
     }
 
+    _ensureGridToggleButton(surface, existingInst = null) {
+        if (!surface?.widgetContainer)
+            return;
+
+        const instanceId = this._getGridToggleButtonInstanceId(surface.monitorIndex);
+
+        if (surface.gridToggleButton) {
+            this._raiseGridToggleButton(surface);
+            return;
+        }
+
+        const gridToggleButton = new Gtk.ToggleButton();
+        gridToggleButton.set_name('ding-widget-grid-toggle-button');
+        gridToggleButton.set_can_focus(false);
+        gridToggleButton.set_focus_on_click(false);
+        gridToggleButton.set_tooltip_text(_('Toggle Widget Grid'));
+        
+        const gridIcon = Gtk.Image.new_from_icon_name('view-grid-symbolic');
+        gridToggleButton.set_child(gridIcon);
+        gridToggleButton.set_active(false);
+
+        gridToggleButton.widgetInstanceId = instanceId;
+
+        gridToggleButton.connect('toggled', (btn) => {
+            surface.grid.widgetGridEnabled = btn.get_active();
+            surface.grid.updateOverlay();
+        });
+
+        surface.widgetContainer.put(gridToggleButton, 0, 0);
+        surface.gridToggleButton = gridToggleButton;
+
+        const inst = existingInst ?? {
+            instanceId,
+            widgetId: '__ding-widget-grid-toggle-button',
+            monitorIndex: surface.monitorIndex,
+            kind: 'chrome',
+            normX: 0,
+            normY: 0,
+            width: 48,
+            height: 48,
+            actor: gridToggleButton,
+            config: {},
+            _isGridToggleButton: true,
+        };
+        inst.actor = gridToggleButton;
+        inst.monitorIndex = surface.monitorIndex;
+        this._instances.set(instanceId, inst);
+
+        if (!existingInst) {
+            const [defaultX, defaultY] =
+                this._getDefaultGridToggleButtonPosition(surface, inst);
+
+            this.setInstanceFrame(instanceId, defaultX, defaultY, inst.width,
+                inst.height
+            );
+        } else {
+            this._positionInstanceActor(inst);
+        }
+
+        this._updateGridToggleButtonVisibility(surface);
+        this._raiseGridToggleButton(surface);
+    }
+
     _updateAddWidgetButtonVisibility(surface, forcedState = null) {
         if (!surface?.addButton)
             return;
@@ -1052,6 +1211,18 @@ const WidgetManager = class {
         surface.addButton.set_sensitive(shouldShow);
     }
 
+    _updateGridToggleButtonVisibility(surface, forcedState = null) {
+        if (!surface?.gridToggleButton)
+            return;
+
+        const shouldShow = typeof forcedState === 'boolean'
+            ? forcedState
+            : Boolean(surface.grid?.isWidgetContainerOnTop?.());
+
+        surface.gridToggleButton.set_visible(shouldShow);
+        surface.gridToggleButton.set_sensitive(shouldShow);
+    }
+
     _raiseAddButton(surface) {
         if (!surface?.addButton || !surface.widgetContainer)
             return;
@@ -1067,6 +1238,22 @@ const WidgetManager = class {
         }
     }
 
+    _raiseGridToggleButton(surface) {
+        if (!surface?.gridToggleButton || !surface.widgetContainer)
+            return;
+
+        const parent = surface.gridToggleButton.get_parent?.();
+        if (!parent || parent !== surface.widgetContainer)
+            return;
+
+        try {
+            surface.gridToggleButton.insert_before(parent, null);
+        } catch (e) {
+            console
+            .error('WidgetManager: failed to raise grid toggle button:', e);
+        }
+    }
+
     _getDefaultAddButtonPosition(surface, inst) {
         const grid = surface.grid;
         if (!grid)
@@ -1074,8 +1261,8 @@ const WidgetManager = class {
 
         const width = grid.normalizedWidth;
         const height = grid.normalizedHeight;
-        const buttonWidth = inst?.width ?? 64;
-        const buttonHeight = inst?.height ?? 64;
+        const buttonWidth = inst?.width ?? 48;
+        const buttonHeight = inst?.height ?? 48;
         const margin = 32;
 
         const direction =
@@ -1095,9 +1282,40 @@ const WidgetManager = class {
         return [x, y];
     }
 
+    _getDefaultGridToggleButtonPosition(surface, inst) {
+        const grid = surface.grid;
+        if (!grid)
+            return [0, 0];
+
+        const addButtonInstanceId =
+            this._getAddButtonInstanceId(surface.monitorIndex);
+
+        const addButtonInst = addButtonInstanceId
+            ? this._instances.get(addButtonInstanceId)
+            : null;
+
+        if (!addButtonInst) {
+            return [0, 0];
+        }
+
+        const width = grid.normalizedWidth;
+        const buttonWidth = inst?.width ?? 48;
+        const buttonHeight = inst?.height ?? 48;
+        const spacing = 16;
+
+        const addButtonX = addButtonInst.normX * grid.normalizedWidth;
+        const addButtonY = addButtonInst.normY * grid.normalizedHeight;
+
+        const x = Math.max(0, Math.min(addButtonX, width - buttonWidth));
+        const desiredY = addButtonY - buttonHeight - spacing;
+        const y = Math.max(0, desiredY);
+
+        return [x, y];
+    }
+
     _detachInstancesWithoutSurface() {
         for (const inst of this._instances.values()) {
-            if (inst?._isAddButton)
+            if (inst?._isAddButton || inst?._isGridToggleButton)
                 continue;
 
             const surface = this._surfaces.get(inst.monitorIndex);
@@ -1133,6 +1351,10 @@ const WidgetManager = class {
                 continue;
 
             this._positionInstanceActor(inst);
+
+            // Use optional chaining because requestRender()
+            // currently exists only on HTML hosts.
+            inst.host?.requestRender?.().catch(e => logError(e));
         }
     }
 
@@ -1298,13 +1520,25 @@ const WidgetManager = class {
 
         const {widgetContainer} = surface;
         const {x, y} = frame;
+        const isMove = !!inst.actor.get_parent();
+
+        if (frame.clamped || isMove) {
+            const [normX, normY] = surface.grid.getNormalizedCoordinates(x, y);
+            const EPSILON = 1e-4;
+
+            if (Math.abs(inst.normX - normX) > EPSILON ||
+                Math.abs(inst.normY - normY) > EPSILON) {
+                inst.normX = normX;
+                inst.normY = normY;
+            }
 
-        if (!inst.actor.get_parent()) {
-            widgetContainer.put(inst.actor, x, y);
-        } else {
-            widgetContainer.move(inst.actor, x, y);
             this._stateChanged();
         }
+
+        if (isMove)
+            widgetContainer.move(inst.actor, x, y);
+        else
+            widgetContainer.put(inst.actor, x, y);
     }
 
     _ensureChrome() {
@@ -1356,32 +1590,54 @@ const WidgetManager = class {
 
         let allocWidth = frame.width;
         const alloc = inst.actor?.get_allocation?.();
-
         if (alloc)
             allocWidth = alloc.width;
 
-        const size = 28; // matches CSS button size
-        const x = frame.x + (allocWidth / 2) - (size / 2);
-        const y = frame.y - size - 6;
+        const size = 28;
+        const gap = 6;
+        const margin = 8;
 
+        const showPrefs = inst.hasPreferences;
+        const buttonCount = showPrefs ? 2 : 1;
+        const totalWidth = buttonCount * size + (buttonCount - 1) * gap;
+
+        const centerX = frame.x + allocWidth / 2;
+        const buttonsX = centerX - totalWidth / 2;
+
+        let yPos;
+        const yPosUp = frame.y - size - margin;
+        const yPosDown = frame.y + frame.height + margin;
+        if (yPosUp < margin) {
+            yPos = yPosDown;
+        } else {
+            yPos = yPosUp;
+        }
+
+        const prefsOldParent = this.prefsButton.get_parent();
+        if (prefsOldParent && prefsOldParent !== widgetContainer)
+            prefsOldParent.remove(this.prefsButton);
+        const closeOldParent = this.closeButton.get_parent();
+        if (closeOldParent && closeOldParent !== widgetContainer)
+            closeOldParent.remove(this.closeButton);
+
+        if (showPrefs) {
+            if (!this.prefsButton.get_parent())
+                widgetContainer.put(this.prefsButton, buttonsX, yPos);
+            else
+                widgetContainer.move(this.prefsButton, buttonsX, yPos);
+            this.prefsButton.show();
+        } else {
+            this.prefsButton.hide();
+        }
+
+        const closeX = showPrefs ? (buttonsX + size + gap) : buttonsX;
         if (!this.closeButton.get_parent())
-            widgetContainer.put(this.closeButton, x, y);
+            widgetContainer.put(this.closeButton, closeX, yPos);
         else
-            widgetContainer.move(this.closeButton, x, y);
-
-        const px = x + size + 6;
-
-        if (!this.prefsButton.get_parent())
-            widgetContainer.put(this.prefsButton, px, y);
-        else
-            widgetContainer.move(this.prefsButton, px, y);
-
+            widgetContainer.move(this.closeButton, closeX, yPos);
         this.closeButton.show();
 
-        if (inst.hasPreferences)
-            this.prefsButton.show();
-        else
-            this.prefsButton.hide();
+        this._raiseChromeButtons(surface);
     }
 
     _detachChrome() {
@@ -1395,6 +1651,23 @@ const WidgetManager = class {
         }
     }
 
+    _raiseChromeButtons(surface) {
+        if (!this._chrome || !surface?.widgetContainer)
+            return;
+
+        for (const btn of this._chrome) {
+            const parent = btn.get_parent?.();
+            if (!parent || parent !== surface.widgetContainer)
+                continue;
+
+            try {
+                btn.insert_before(parent, null);
+            } catch (e) {
+                console.error('WidgetManager: failed to raise chrome button:', e);
+            }
+        }
+    }
+
     _openPreferencesForSelectedInstance() {
         const selectedId = this._selectedInstanceId;
         const inst = selectedId
@@ -1430,6 +1703,7 @@ const WidgetManager = class {
         }
 
         this._raiseAddButton(surface);
+        this._raiseGridToggleButton(surface);
     }
 
     _getWidgetKind(_widgetId) {
@@ -1743,9 +2017,6 @@ const WidgetManager = class {
     _addActions() {
         const addWidgetAction = Gio.SimpleAction.new('addWidget', null);
         addWidgetAction.connect('activate', () => {
-            // Ensure widget layers are visible before adding a widget.
-            this._desktopManager.windowManager?.raiseWidgetLayers();
-
             const parentWindow =
                 this._desktopManager.mainApp.get_active_window();
 
@@ -1766,11 +2037,57 @@ const WidgetManager = class {
                 }
             }
 
+            if (monitorIndex === null)
+                return;
+
+            // Ensure widget layers are visible before adding a widget.
+            this._desktopManager.windowManager?.raiseWidgetLayers();
+
             this.openAddWidgetDialog(parentWindow, monitorIndex)
                 .catch(logError);
         });
         this._desktopManager.mainApp.add_action(addWidgetAction);
 
+        const showGridAction = Gio.SimpleAction.new('toggleWidgetGrid', null);
+        showGridAction.connect('activate', () => {
+            const parentWindow =
+                this._desktopManager.mainApp.get_active_window();
+
+            let monitorIndex = null;
+
+            if (parentWindow) {
+                const surface = parentWindow.get_surface();
+                const display = surface?.get_display?.();
+                const monitor = display?.get_monitor_at_surface?.(surface);
+                const monitors = display?.get_monitors?.();
+                const count = monitors?.get_n_items?.() ?? 0;
+
+                for (let i = 0; i < count; i++) {
+                    if (monitors.get_item?.(i) === monitor) {
+                        monitorIndex = i;
+                        break;
+                    }
+                }
+            }
+
+            if (monitorIndex === null)
+                return;
+
+            let gridToggleButton = null;
+            const instanceId =
+                this._getGridToggleButtonInstanceId(monitorIndex);
+
+            const inst = instanceId ? this._instances.get(instanceId) : null;
+            gridToggleButton = inst?.actor ?? null;
+
+            if (!gridToggleButton)
+                return;
+
+            // Ensure widget layers are visible before showingt widget grid.
+            this._desktopManager.windowManager?.raiseWidgetLayers();
+            gridToggleButton?.activate();
+        });
+        this._desktopManager.mainApp.add_action(showGridAction);
 
         const closeWidget = Gio.SimpleAction.new('closeWidget', null);
         closeWidget.connect('activate', this.deleteSelectedInstance.bind(this));
